@@ -72,7 +72,7 @@ stan_aov <- function(formula, data, projections = FALSE,
         if (projections) 
           fit$projections <- proj(fit)
         fit$call <- Call
-        fit$modeling_function <- "stan_aov"
+        fit$stan_function <- "stan_aov"
         return(fit)
     } else { # nocov start
       
