@@ -117,6 +117,11 @@ stan_biglm.fit <- function(b, R, SSR, N, xbar, ybar, s_y, has_intercept = TRUE,
     stanfit <- do.call(sampling, sampling_args)
   }
   check_stanfit(stanfit)
+  if (K == 1)
+    stanfit@sim$samples <- lapply(stanfit@sim$samples, FUN = function(x) {
+      x$`R2[1]` <- (x$`R2[1]`)^2
+      return(x)
+    })
   new_names <- c(if (has_intercept) "(Intercept)", cn, "sigma", 
                  if (prior_PD == 0) "log-fit_ratio", 
                  "R2", "mean_PPD", "log-posterior")
