@@ -1,4 +1,5 @@
 # Part of the rstanarm package for estimating model parameters
+# Copyright (C) 2016 Simon N. Wood
 # Copyright (C) 2015, 2016, 2017 Trustees of Columbia University
 # 
 # This program is free software; you can redistribute it and/or
@@ -15,8 +16,8 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
-#' Bayesian generalized linear additive models with group-specific terms via
-#' Stan
+#' Bayesian generalized linear additive models with optional group-specific
+#' terms via Stan
 #' 
 #' Bayesian inference for GAMMs with flexible priors.
 #' 
@@ -26,8 +27,10 @@
 #' @templateVar pkgfun gamm4
 #' @template return-stanreg-object
 #' @template see-also
+#' @template args-prior_intercept
 #' @template args-priors
 #' @template args-prior_aux
+#' @template args-prior_smooth
 #' @template args-prior_PD
 #' @template args-algorithm
 #' @template args-adapt_delta
@@ -49,33 +52,30 @@
 #'   more information about the default arguments.
 #'
 #' @details The \code{stan_gamm4} function is similar in syntax to 
-#'   \code{\link[gamm4]{gamm4}} in the \pkg{gamm4} package, which accepts a syntax 
-#'   that is similar to (but not quite as extensive as) that for 
-#'   \code{\link[mgcv]{gamm}} in the \pkg{mgcv} package and converts 
-#'   it internally into the syntax accepted by \code{\link[lme4]{glmer}} in the
-#'   \pkg{lme4} package. But rather than performing (restricted) maximum likelihood 
-#'   estimation, the \code{stan_gamm4} function utilizes MCMC to perform Bayesian 
+#'   \code{\link[gamm4]{gamm4}} in the \pkg{gamm4} package. But rather than performing 
+#'   (restricted) maximum likelihood estimation with the \pkg{lme4} package,
+#'   the \code{stan_gamm4} function utilizes MCMC to perform Bayesian 
 #'   estimation. The Bayesian model adds priors on the common regression 
-#'   coefficients (in the same way as \code{\link{stan_glm}}) and priors on the 
-#'   terms of a decomposition of the covariance matrices of the group-specific 
-#'   parameters, including the smooths. Estimating these models via MCMC avoids
+#'   coefficients (in the same way as \code{\link{stan_glm}}), priors on the 
+#'   standard deviations of the smooth terms, and a prior on the decomposition
+#'   of the covariance matrices of any group-specific parameters (as in 
+#'   \code{\link{stan_glmer}}). Estimating these models via MCMC avoids
 #'   the optimization issues that often crop up with GAMMs and provides better
 #'   estimates for the uncertainty in the parameter estimates. 
 #'   
 #'   See \code{\link[gamm4]{gamm4}} for more information about the model
 #'   specicification and \code{\link{priors}} for more information about the
-#'   priors. If \code{random = NULL}, the output is a subset of that produced by
-#'   \code{\link[mgcv]{gam}} in the sense that there are several estimated components
-#'   for each smooth term. However, the parameterization used to estimate the model
-#'   is different and corresponds to the parameterization in 
-#'   \code{\link[gamm4]{gamm4}} where is smooth term is decomposed into a linear
-#'   and a non-linear part. If \code{prior} is not \code{NULL}, then the number 
-#'   of parameters to place priors on is equal to the number of linear terms in
-#'   the \code{formula}. The prior on the non-linear part of each smooth term is
-#'   handled by the \code{\link{decov}} function. If \code{random} is not \code{NULL},
-#'   then there are additional group-specific terms whose priors are also handled
-#'   by the \code{\link{decov}} function and whose posterior medians can be extracted
-#'   by calling \code{\link[lme4]{ranef}}.
+#'   priors on the main coefficients. The \code{formula} should include at least
+#'   one smooth term, which can be specified in any way that is supported by the
+#'   \code{\link[mgcv]{jagam}} function in the \pkg{mgcv} package. The 
+#'   \code{prior_smooth} argument should be used to specify a prior on the unknown
+#'   standard deviations that govern how smooth the smooth function is. The
+#'   \code{prior_covariance} argument can be used to specify the prior on the
+#'   components of the covariance matrix for any (optional) group-specific terms.
+#'   The \code{\link[gamm4]{gamm4}} function in the \pkg{gamm4} package uses
+#'   group-specific terms to implement the departure from linearity in the smooth
+#'   terms, but that is not the case for \code{stan_gamm4} where the group-specific
+#'   terms are exactly the same as in \code{\link{stan_glmer}}.
 #'   
 #'   The \code{plot_nonlinear} function creates a ggplot object with one facet for
 #'   each smooth function specified in the call to \code{stan_gamm4} in the case
@@ -97,6 +97,9 @@
 #' Software}. \strong{14}(14), 1--22. 
 #' \url{https://www.jstatsoft.org/article/view/v014i14}
 #' 
+#' @seealso The vignette for \code{stan_glmer}, which also discusses
+#'   \code{stan_gamm4}.
+#' 
 #' @examples
 #' # from example(gamm4, package = "gamm4"), prefixing gamm4() call with stan_
 #' \donttest{
@@ -106,103 +109,153 @@
 #' dat$y <- dat$y + model.matrix(~ fac - 1) %*% rnorm(20) * .5
 #'
 #' br <- stan_gamm4(y ~ s(x0) + x1 + s(x2), data = dat, random = ~ (1 | fac), 
-#'                  QR = TRUE, chains = 1, iter = 200) # for example speed
+#'                  chains = 1, iter = 200) # for example speed
 #' print(br)
 #' plot_nonlinear(br)
 #' plot_nonlinear(br, smooths = "s(x0)", alpha = 2/3)
 #' }
 #' 
-#' @importFrom lme4 getME
-stan_gamm4 <- function(formula, random = NULL, family = gaussian(), data, 
-                       weights = NULL, subset = NULL, na.action, knots = NULL, 
-                       drop.unused.levels = TRUE, ..., 
-                       prior = normal(), prior_intercept = normal(),
-                       prior_aux = cauchy(0, 5),
-                       prior_covariance = decov(), prior_PD = FALSE, 
-                       algorithm = c("sampling", "meanfield", "fullrank"), 
-                       adapt_delta = NULL, QR = FALSE, sparse = FALSE) {
-
-  mc <- match.call(expand.dots = FALSE)
+stan_gamm4 <-
+  function(formula,
+           random = NULL,
+           family = gaussian(),
+           data,
+           weights = NULL,
+           subset = NULL,
+           na.action,
+           knots = NULL,
+           drop.unused.levels = TRUE,
+           ...,
+           prior = normal(),
+           prior_intercept = normal(),
+           prior_smooth = exponential(autoscale = FALSE),
+           prior_aux = exponential(),
+           prior_covariance = decov(),
+           prior_PD = FALSE,
+           algorithm = c("sampling", "meanfield", "fullrank"),
+           adapt_delta = NULL,
+           QR = FALSE,
+           sparse = FALSE) {
+    
   data <- validate_data(data, if_missing = list())
   family <- validate_family(family)
-  glmod <- suppressWarnings(gamm4_to_glmer(formula, random, family, data, weights, 
-                                           subset, na.action, knots, drop.unused.levels))
-  colnames(glmod$X) <- gsub("^X\\.0", "", colnames(glmod$X))
-  colnames(glmod$X) <- gsub("Fx1$",   "", colnames(glmod$X))
-  X <- glmod$X
-  y <- glmod$fr[, as.character(glmod$formula[2L])]
-  if (is.matrix(y) && ncol(y) == 1L) y <- as.vector(y)
-  offset <- model.offset(glmod$fr) %ORifNULL% double(0)
-  weights <- validate_weights(glmod$fr$weights)
   
-  if (is.null(prior))
-    prior <- list()
-  if (is.null(prior_intercept)) 
-    prior_intercept <- list()
-  if (is.null(prior_aux)) 
-    prior_aux <- list()
-
-  group <- glmod$reTrms
-  group$decov <- prior_covariance
+  if (!is.null(random)) {
+    fake.formula <- as.character(mgcv::interpret.gam(formula)$fake.formula)
+    form <- paste(fake.formula[2], fake.formula[1], fake.formula[3],
+                  "+", random[2], collapse = " ")
+    glmod <- lme4::glFormula(as.formula(form), data, family = gaussian,
+                             subset, weights, na.action,
+                             control = make_glmerControl())
+    data <- glmod$fr
+    weights <- validate_weights(glmod$fr$weights)
+  }
+  else {
+    weights <- validate_weights(weights)
+    glmod <- NULL
+  }
+  
+  if (family$family == "binomial") {
+    data$temp_y <- rep(1, NROW(data)) # work around jagam bug
+    temp_formula <- update(formula, temp_y ~ .)
+    jd <- mgcv::jagam(formula = temp_formula, family = gaussian(), data = data,
+                      file = tempfile(fileext = ".jags"), weights = NULL,
+                      na.action = na.action, offset = NULL, knots = knots,
+                      drop.unused.levels = drop.unused.levels, diagonalize = TRUE)
+    
+    if (!is.null(random)) {
+      y <- data[, as.character(formula[2L])]
+    } else {
+      y <- eval(formula[[2L]], data)
+    }
+    
+    if (binom_y_prop(y, family, weights)) {
+      y1 <- as.integer(as.vector(y) * weights)
+      y <- cbind(y1, y0 = weights - y1)
+      weights <- double(0)
+    }
+  } else {
+    jd <- mgcv::jagam(formula = formula, family = gaussian(), data = data,
+                      file = tempfile(fileext = ".jags"), weights = NULL,
+                      na.action = na.action, offset = NULL, knots = knots,
+                      drop.unused.levels = drop.unused.levels, diagonalize = TRUE)
+    y <- jd$jags.data$y
+  }
+  # there is no offset allowed by gamm4::gamm4
+  offset <- validate_offset(as.vector(model.offset(jd$pregam$model)), y = y)
+  X <- jd$jags.data$X
+  mark <- which(colnames(X) != "")
+  colnames(X) <- colnames(jd$pregam$X) <- jd$pregam$term.names
+  S <- lapply(jd$pregam$smooth, FUN = function(s) {
+    ranks <- s$rank
+    start <- s$first.para
+    out <- list()
+    for (r in seq_along(ranks)) {
+      end <- start + ranks[r] - 1L
+      out[[r]] <- X[,start:end, drop = FALSE]
+      start <- end + 1L
+    }
+    return(out)
+  })
+  if (any(sapply(S, length) > 1)) S <- unlist(S, recursive = FALSE)
+  names(S) <- names(jd$pregam$sp)
+  X <- X[,mark, drop = FALSE]
+  X <- c(list(X), S)
+  
+  if (is.null(prior)) prior <- list()
+  if (is.null(prior_intercept)) prior_intercept <- list()
+  if (is.null(prior_aux)) prior_aux <- list()
+  if (is.null(prior_smooth)) prior_smooth <- list()
+  
+  if (is.null(random)) {
+    group <- list()
+    prior_covariance <- list()
+  }
+  else {
+    group <- glmod$reTrms
+    group$decov <- prior_covariance
+  }
   algorithm <- match.arg(algorithm)
   
   stanfit <- stan_glm.fit(x = X, y = y, weights = weights,
                           offset = offset, family = family,
                           prior = prior, prior_intercept = prior_intercept,
-                          prior_aux = prior_aux, prior_PD = prior_PD, 
-                          algorithm = algorithm, adapt_delta = adapt_delta,
-                          group = group, QR = QR, ...)
-
-  if (nrow(glmod$raw_X) < ncol(glmod$raw_X)) {
-    warning("cannot reparameterize output, returning a stanfit object with reduced functionaliy")
-    return(stanfit)
+                          prior_aux = prior_aux, prior_smooth = prior_smooth,
+                          prior_PD = prior_PD, algorithm = algorithm, 
+                          adapt_delta = adapt_delta, group = group, QR = QR, ...)
+  if (family$family == "Beta regression") family$family <- "beta"
+  X <- do.call(cbind, args = X)
+  if (is.null(random)) Z <- Matrix::Matrix(nrow = NROW(y), ncol = 0, sparse = TRUE)
+  else {
+    Z <- pad_reTrms(Ztlist = group$Ztlist, cnms = group$cnms, 
+                    flist = group$flist)$Z
+    colnames(Z) <- b_names(names(stanfit), value = TRUE)
   }
-  # Convert back to gam parameterization
-  genuine <- lme4::findbars(random)
-  nrows <- sapply(group$Ztlist, FUN = nrow)
-  Z <- pad_reTrms(Ztlist = group$Ztlist, cnms = group$cnms, 
-                  flist = group$flist)$Z
-  colnames(Z) <- b_names(names(stanfit), value = TRUE)
-  if (length(genuine)) {
-    mark <- grep(" Xr", colnames(Z), fixed = TRUE)
-    Z_nl <- Z[,mark, drop = FALSE]
-  }
-  else Z_nl <- Z
-  XZ <- if (getRversion() < "3.2.0") cBind(X, Z_nl) else cbind2(X, Z_nl)
-  arr <- as.array(stanfit)
-  XtX <- crossprod(glmod$raw_X)
-  for (chain in 1:ncol(stanfit)) {
-    end <- tail(grep("b[", names(stanfit@sim$samples[[chain]]), 
-                     fixed = TRUE, value = FALSE), 1)
-    if (length(genuine)) {
-      for (start in rev(grep(" Xr", colnames(Z), fixed = TRUE, invert = TRUE))) {
-        stanfit@sim$samples[[chain]][end] <- stanfit@sim$samples[[chain]][start]
-        names(stanfit@sim$samples[[chain]])[end] <-
-        names(stanfit@sim$samples[[chain]])[start]  
-        end <- end - 1L
-      }
-    }
-    eta <- linear_predictor(arr[, chain, colnames(XZ)], x = XZ)
-    beta <- qr.solve(XtX, t(glmod$raw_X) %*% t(eta))
-    for (j in 1:nrow(beta)) {
-      stanfit@sim$samples[[chain]][[j]] <- beta[j,]
-      names(stanfit@sim$samples[[chain]])[j] <- rownames(beta)[j]
-    }
-  }
-  X <- glmod$raw_X
-  stanfit@par_dims$beta <- ncol(X) - stanfit@par_dims$alpha
-  if (length(genuine)) Z <- Z[,-mark,drop = FALSE]
-  else Z <- Matrix::Matrix(nrow = nrow(Z), ncol = 0, sparse = TRUE)
-  stanfit@par_dims$b <- ncol(Z)
-  XZ <- if (getRversion() < "3.2.0") cBind(X, Z) else cbind2(X, Z)
-  stanfit@sim$fnames_oi[1:ncol(XZ)] <- colnames(XZ)
+  if (getRversion() < "3.2.0") XZ <- cBind(X, Z) 
+  else XZ <- cbind2(X, Z)
+  
+  # make jam object with point estimates, see ?mgcv::sim2jam
+  mat <- as.matrix(stanfit)
+  mark <- 1:ncol(X)
+  jd$pregam$Vp <- cov(mat[,mark, drop = FALSE])
+  jd$pregam$coefficients <- colMeans(mat[,mark, drop = FALSE])
+  jd$pregam$sig2 <- if ("sigma" %in% colnames(mat)) mean(mat[,"sigma"]) else 1
+  eta <- X %*% t(mat[,mark,drop = FALSE])
+  mu <- rowMeans(family$linkinv(eta))
+  eta <- rowMeans(eta)
+  w <- as.numeric(jd$pregam$w * family$mu.eta(eta) ^ 2 / family$variance(mu))
+  XWX <- t(X) %*% (w * X)
+  jd$pregam$edf <- rowSums(jd$pregam$Vp * t(XWX)) / jd$pregam$sig2
+  class(jd$pregam) <- c("jam", "gam")
   fit <- nlist(stanfit, family, formula, offset, weights, 
-               x = XZ, y = y, data, terms = NULL, model = NULL, 
+               x = XZ, y = y, data, terms = jd$pregam$terms, 
+               model = if (is.null(random)) jd$pregam$model else glmod$fr,
                call = match.call(expand.dots = TRUE),
-               algorithm, glmod = glmod, 
-               modeling_function = "stan_gamm4")
+               algorithm, glmod = glmod,
+               stan_function = "stan_gamm4")
   out <- stanreg(fit)
-  class(out) <- c(class(out), "gamm4", "lmerMod")
+  out$jam <- jd$pregam
+  class(out) <- c(class(out), "gamm4", if (!is.null(glmod)) "lmerMod")
   return(out)
 }
 
@@ -230,14 +283,17 @@ plot_nonlinear <- function(x, smooths, ...,
   validate_stanreg_object(x)
   if (!is(x, "gamm4"))
     stop("Plot only available for models fit using the stan_gamm4 function.")
-  
+  on.exit(message("try plot(x$jam) instead"))
   scheme <- bayesplot::color_scheme_get()
   
   XZ <- x$x
   XZ <- XZ[,!grepl("_NEW_", colnames(XZ), fixed = TRUE)]
-  labels <- sapply(x$glmod$smooths, "[[", "label")
-  xnames <- sapply(x$glmod$smooths, "[[", "vn")
+  labels <- sapply(x$jam$smooth, "[[", "label")
+  xnames <- sapply(x$jam$smooth, "[[", "vn")
+  names(x$jam$smooth) <- labels
   names(xnames) <- labels
+  fs <- sapply(x$jam$smooth, FUN = "inherits", what = "fs.interaction")
+  
   if (!missing(smooths)) {
     found <- smooths %in% labels
     if (all(!found)) {
@@ -249,16 +305,20 @@ plot_nonlinear <- function(x, smooths, ...,
               paste(smooths[!found], collapse = ", "))
     }
     labels <- smooths[found]
+    fs <- fs[found]
     if (!is.matrix(xnames)) xnames <- xnames[found]
   }
+  else smooths <- 1:length(labels)
   
   B <- as.matrix(x)[, colnames(XZ), drop = FALSE]
-  original <- x$glmod$model
+  original <- x$jam$model
   
   bivariate <- any(grepl(",", labels, fixed = TRUE))
-  if (bivariate) {
-    if (length(labels) > 1)
+  if (bivariate && !any(fs)) {
+    if (length(labels) > 1) {
+      on.exit(NULL)
       stop("Multivariate functions can only be plotted one at a time; specify 'smooths'.")
+    }
     if (length(xnames) > 2)
       stop("Only univariate and bivariate functions can be plotted currently.")
     xrange <- range(original[, xnames[1]])
@@ -267,12 +327,12 @@ plot_nonlinear <- function(x, smooths, ...,
                       seq(from = yrange[1], to = yrange[2], length.out = 100))
     colnames(xz) <- xnames[1:2]
     plot_data <- data.frame(x = xz[, 1], y = xz[, 2])
-    for (i in colnames(original)) {
-      if (i %in% colnames(xz)) next
-      xz[[i]] <- 1
-    }
-    XZ <- mgcv::predict.gam(mgcv::gam(formula(x), data = x$data), 
-                            newdata = xz, type = "lpmatrix")
+    nd <- original
+    nd <- nd[sample(nrow(xz), size = nrow(xz), replace = TRUE), ]
+    nd[[xnames[1]]] <- xz[[xnames[1]]]
+    nd[[xnames[2]]] <- xz[[xnames[2]]]
+    requireNamespace("mgcv", quietly = TRUE)
+    XZ <- predict(x$jam, newdata = nd, type = "lpmatrix")
     incl <- grepl(labels, colnames(B), fixed = TRUE)
     b <- B[, incl, drop = FALSE]
     xz <- XZ[, grepl(labels, colnames(XZ), fixed = TRUE), drop = FALSE]
@@ -288,20 +348,48 @@ plot_nonlinear <- function(x, smooths, ...,
     )
   }
   
-  df_list <- lapply(labels, FUN = function(term) {
-    incl <- grepl(term, colnames(B), fixed = TRUE)
-    xz <- XZ[, incl, drop = FALSE]
+  df_list <- lapply(x$jam$smooth[smooths], FUN = function(s) {
+    incl <- s$first.para:s$last.para
     b <- B[, incl, drop = FALSE]
-    x <- original[, xnames[term]]
-    xz <- xz[order(x), , drop=FALSE]
-    f <- linear_predictor.matrix(b, xz)
-    data.frame(
-      predictor = sort(x),
-      lower  = apply(f, 2, quantile, probs = (1 - prob) / 2),
-      upper  = apply(f, 2, quantile, probs = prob + (1 - prob) / 2),
-      middle = apply(f, 2, median),
-      term = term
-    )
+    if (inherits(s, "fs.interaction")) { # see mgcv:::plot.fs.interaction
+      xx <- original[,s$base$term]
+      fac <- original[,s$fterm]
+      out <- by(data.frame(fac, xx), list(fac), FUN = function(df) {
+        df <- df[order(df[,2]),]
+        names(df) <- c(s$fterm, s$base$term)
+        xz <- mgcv::PredictMat(s, df)
+        f <- linear_predictor.matrix(b, xz)
+        data.frame(
+          predictor = df[,2],
+          lower  = apply(f, 2, quantile, probs = (1 - prob) / 2),
+          upper  = apply(f, 2, quantile, probs = prob + (1 - prob) / 2),
+          middle = apply(f, 2, median),
+          term = paste(s$label, df[,1], sep = ".")
+        )
+      })
+      do.call(rbind, args = out)
+    }
+    else {
+      xz <- XZ[, incl, drop = FALSE]
+      x <- original[, s$term]
+      ord <- order(x)
+      x <- x[ord]
+      xz <- xz[ord, , drop=FALSE]
+      if (!is.null(s$by.level)) {
+        fac <- original[,s$by][ord]
+        mark <- fac == s$by.level
+        x <- x[mark]
+        xz <- xz[mark, , drop = FALSE]
+      }
+      f <- linear_predictor.matrix(b, xz)
+      data.frame(
+        predictor = x,
+        lower  = apply(f, 2, quantile, probs = (1 - prob) / 2),
+        upper  = apply(f, 2, quantile, probs = prob + (1 - prob) / 2),
+        middle = apply(f, 2, median),
+        term = s$label
+      )
+    }
   })
   plot_data <- do.call(rbind, df_list)
   
@@ -310,7 +398,8 @@ plot_nonlinear <- function(x, smooths, ...,
     facet_args[["scales"]] <- "free"
   if (is.null(facet_args[["strip.position"]]))
     facet_args[["strip.position"]] <- "left"
-  
+
+  on.exit(NULL)  
   ggplot(plot_data, aes_(x = ~ predictor)) + 
     geom_ribbon(aes_(ymin = ~ lower, ymax = ~ upper), 
                 fill = scheme[[1]], color = scheme[[2]],
