@@ -27,8 +27,10 @@
 #' @templateVar pkgfun glmer
 #' @template return-stanreg-object
 #' @template see-also
+#' @template args-prior_intercept
 #' @template args-priors
 #' @template args-prior_aux
+#' @template args-prior_covariance
 #' @template args-prior_PD
 #' @template args-algorithm
 #' @template args-adapt_delta
@@ -50,8 +52,6 @@
 #'   \code{"meanfield"} or \code{"fullrank"}). For \code{stan_lmer} and 
 #'   \code{stan_glmer.nb}, \code{...} should also contain all relevant arguments
 #'   to pass to \code{stan_glmer} (except \code{family}).
-#' @param prior_covariance Cannot be \code{NULL}; see \code{\link{decov}} for
-#'   more information about the default arguments.
 #'
 #' @details The \code{stan_glmer} function is similar in syntax to 
 #'   \code{\link[lme4]{glmer}} but rather than performing (restricted) maximum 
@@ -80,15 +80,25 @@
 #' 
 #' @importFrom lme4 glFormula
 #' @importFrom Matrix Matrix t cBind
-stan_glmer <- function(formula, data = NULL, family = gaussian, 
-                       subset, weights, 
-                       na.action = getOption("na.action", "na.omit"), 
-                       offset, contrasts = NULL, ...,
-                       prior = normal(), prior_intercept = normal(),
-                       prior_aux = cauchy(0, 5),
-                       prior_covariance = decov(), prior_PD = FALSE, 
-                       algorithm = c("sampling", "meanfield", "fullrank"), 
-                       adapt_delta = NULL, QR = FALSE, sparse = FALSE) {
+stan_glmer <- 
+  function(formula,
+           data = NULL,
+           family = gaussian,
+           subset,
+           weights,
+           na.action = getOption("na.action", "na.omit"),
+           offset,
+           contrasts = NULL,
+           ...,
+           prior = normal(),
+           prior_intercept = normal(),
+           prior_aux = exponential(),
+           prior_covariance = decov(),
+           prior_PD = FALSE,
+           algorithm = c("sampling", "meanfield", "fullrank"),
+           adapt_delta = NULL,
+           QR = FALSE,
+           sparse = FALSE) {
   
   call <- match.call(expand.dots = TRUE)
   mc <- match.call(expand.dots = FALSE)
@@ -124,7 +134,9 @@ stan_glmer <- function(formula, data = NULL, family = gaussian,
                           prior_aux = prior_aux, prior_PD = prior_PD, 
                           algorithm = algorithm, adapt_delta = adapt_delta,
                           group = group, QR = QR, sparse = sparse, ...)
-
+  if (family$family == "Beta regression") family$family <- "beta"
+  sel <- apply(X, 2L, function(x) !all(x == 1) && length(unique(x)) < 2)
+  X <- X[ , !sel, drop = FALSE]
   Z <- pad_reTrms(Ztlist = group$Ztlist, cnms = group$cnms, 
                   flist = group$flist)$Z
   colnames(Z) <- b_names(names(stanfit), value = TRUE)
@@ -133,7 +145,7 @@ stan_glmer <- function(formula, data = NULL, family = gaussian,
                x = if (getRversion() < "3.2.0") cBind(X, Z) else cbind2(X, Z), 
                y = y, data, call, terms = NULL, model = NULL, 
                na.action = attr(glmod$fr, "na.action"), contrasts, algorithm, glmod, 
-               modeling_function = "stan_glmer")
+               stan_function = "stan_glmer")
   out <- stanreg(fit)
   class(out) <- c(class(out), "lmerMod")
   
@@ -143,22 +155,23 @@ stan_glmer <- function(formula, data = NULL, family = gaussian,
 
 #' @rdname stan_glmer
 #' @export
-stan_lmer <- function(formula,
-                      data = NULL,
-                      subset,
-                      weights,
-                      na.action = getOption("na.action", "na.omit"),
-                      offset,
-                      contrasts = NULL,
-                      ...,
-                      prior = normal(),
-                      prior_intercept = normal(),
-                      prior_aux = cauchy(0, 5),
-                      prior_covariance = decov(),
-                      prior_PD = FALSE,
-                      algorithm = c("sampling", "meanfield", "fullrank"),
-                      adapt_delta = NULL,
-                      QR = FALSE) {
+stan_lmer <- 
+  function(formula,
+           data = NULL,
+           subset,
+           weights,
+           na.action = getOption("na.action", "na.omit"),
+           offset,
+           contrasts = NULL,
+           ...,
+           prior = normal(),
+           prior_intercept = normal(),
+           prior_aux = exponential(),
+           prior_covariance = decov(),
+           prior_PD = FALSE,
+           algorithm = c("sampling", "meanfield", "fullrank"),
+           adapt_delta = NULL,
+           QR = FALSE) {
   if ("family" %in% names(list(...)))
     stop("'family' should not be specified.")
   mc <- call <- match.call(expand.dots = TRUE)
@@ -169,7 +182,7 @@ stan_lmer <- function(formula,
   mc$family <- "gaussian"
   out <- eval(mc, parent.frame())
   out$call <- call
-  out$modeling_function <- "stan_lmer"
+  out$stan_function <- "stan_lmer"
   return(out)
 }
 
@@ -179,23 +192,25 @@ stan_lmer <- function(formula,
 #' @param link For \code{stan_glmer.nb} only, the link function to use. See 
 #'   \code{\link{neg_binomial_2}}.
 #' 
-stan_glmer.nb <- function(formula,
-                          data = NULL,
-                          subset,
-                          weights,
-                          na.action = getOption("na.action", "na.omit"),
-                          offset,
-                          contrasts = NULL,
-                          link = "log",
-                          ...,
-                          prior = normal(),
-                          prior_intercept = normal(),
-                          prior_aux = cauchy(0, 5),
-                          prior_covariance = decov(),
-                          prior_PD = FALSE,
-                          algorithm = c("sampling", "meanfield", "fullrank"),
-                          adapt_delta = NULL,
-                          QR = FALSE) {
+stan_glmer.nb <- 
+  function(formula,
+           data = NULL,
+           subset,
+           weights,
+           na.action = getOption("na.action", "na.omit"),
+           offset,
+           contrasts = NULL,
+           link = "log",
+           ...,
+           prior = normal(),
+           prior_intercept = normal(),
+           prior_aux = exponential(),
+           prior_covariance = decov(),
+           prior_PD = FALSE,
+           algorithm = c("sampling", "meanfield", "fullrank"),
+           adapt_delta = NULL,
+           QR = FALSE) {
+    
   if ("family" %in% names(list(...)))
     stop("'family' should not be specified.")
   mc <- call <- match.call(expand.dots = TRUE)
@@ -206,6 +221,6 @@ stan_glmer.nb <- function(formula,
   mc$family <- neg_binomial_2(link = link)
   out <- eval(mc, parent.frame())
   out$call <- call
-  out$modeling_function <- "stan_glmer.nb"
+  out$stan_function <- "stan_glmer.nb"
   return(out)
 }
